@@ -49,6 +49,7 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         if(tools){
             this.addTool.apply(this, tools);
         }
+        this.useMenuSearch = this.initialConfig.useMenuSearch;
     },
 
     // private
@@ -87,14 +88,54 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         var mc = this.menuBWrap.wrap({
             cls: "x-window-mc ux-start-menu-bwrap"
         });
-
-        this.menuPanel = this.menuBWrap.createChild({
+        this.mainMenuPanel = this.menuBWrap.createChild({
             tag: "div",
-            cls: "x-panel x-border-panel ux-start-menu-apps-panel"
+            cls: "x-panel",
+            style: {float: 'left'},
         });
+        this.menuPanel = this.mainMenuPanel.createChild({
+            tag: "div",
+            cls: "x-panel ux-start-menu-apps-panel"
+        });
+
+        // Формирование поля для поиска и панели для него
+        if (this.useMenuSearch) {
+            this.menuSearchField = new Ext.app.form.SearchField({
+                emptyText: 'Поиск',
+                hideLabel: true,
+                triggerClass: 'x-form-search-trigger',
+                startMenu: this,
+                enableKeyEvents: true,
+                onTrigger1Click: function () {
+                    // Очистка поля для поиска и возвращение исходного меню
+                    this.setValue('');
+                    this.triggers[0].hide();
+                    this.startMenu.closeAllFloatMenu();
+                    this.startMenu.clearMenuSearchFilter();
+                },
+                onTrigger2Click: function () {
+                    // Установка фильтрации элементов меню по тексту в поле
+                    var filterText = this.getValue(),
+                        trigger = this.triggers[0];
+                    filterText ? trigger.show() : trigger.hide();
+                    this.startMenu.showFilteredStartMenuItems(filterText);
+                },
+            });
+            this.menuSearchField.on('keydown', function () {
+                this.startMenu.closeAllFloatMenu();
+            });
+            this.menuSearchPanel = new Ext.FormPanel({
+                cls: 'x-menu-list-item',
+                items: [this.menuSearchField],
+                style: {'background-color': 'white'},
+            });
+            this.menuSearchPanel.render(this.mainMenuPanel);
+        }
+
         this.toolsPanel = this.menuBWrap.createChild({
             tag: "div",
-            cls: "x-panel x-border-panel ux-start-menu-tools-panel"
+            cls: "x-panel ux-start-menu-tools-panel",
+            style: {float: 'left', height: '100%'},
         });
 
         var bwrap = ml.wrap({cls: "x-window-bwrap"});
@@ -140,13 +181,11 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
 
         this.menuPanel.setStyle({
             padding: '2px',
-            position: 'absolute',
             overflow: 'auto'
         });
 
         this.toolsPanel.setStyle({
             padding: '2px 4px 2px 2px',
-            position: 'absolute',
             overflow: 'auto'
         });
 
@@ -176,6 +215,10 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         this.parentMenu = parentMenu;
         if(!this.el){
             this.render();
+        } else {
+            if (this.useMenuSearch) {
+                this.menuSearchField.clear();
+            }
         }
 
         this.fireEvent("beforeshow", this);
@@ -185,14 +228,21 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         posArray[1] = posArray[1] < 0 ? el.getHeight() : posArray[1];
         this.showAt(posArray ,parentMenu, false);
 
-        var tPanelWidth = this.toolsPanelWidth;
-        var box = this.menuBWrap.getBox();
-        this.menuPanel.setWidth(box.width-tPanelWidth);
-        this.menuPanel.setHeight(box.height);
+        var box = this.menuBWrap.getBox(),
+            leftMenuWidth = box.width - this.toolsPanelWidth,
+            menuPanelHeight = box.height;
 
-        this.toolsPanel.setWidth(tPanelWidth);
-        this.toolsPanel.setX(box.x+box.width-tPanelWidth);
-        this.toolsPanel.setHeight(box.height);
+        if (this.useMenuSearch) {
+            this.menuSearchPanel.setWidth(leftMenuWidth);
+            this.menuSearchField.itemCt.setStyle({margin: '2px'});
+            menuPanelHeight -= this.menuSearchPanel.getHeight();
+            // Нужно для установки всей ширины
+            this.menuSearchField.wrap.setWidth('100%');
+            this.menuSearchField.triggers[1].show();
+        }
+        this.menuPanel.setHeight(menuPanelHeight);
+        this.menuPanel.setWidth(leftMenuWidth);
+        this.toolsPanel.setWidth(this.toolsPanelWidth);
     },
 
     addTool : function(){
@@ -253,7 +303,109 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         this.title = title;
         this.header.child('span').update(title);
         return this;
-    }
+    },
+
+    /**
+     * Закрывает все выпадающие меню, чтобы после фильтрации не возникали баги,
+     * связанные с его расположением
+     * @param complexMenuItem Элемент меню с вложенными элементами.
+     * Если параметр не указан, выбирается основное меню.
+     */
+    closeAllFloatMenu: function (complexMenuItem) {
+        var menuItem = complexMenuItem || this;
+        menuItem.items.each(function (item) {
+            var menu = item.menu;
+
+            if (menu) {
+                if (menu.ul && menu.ul.isVisible()) {
+                    item.deactivate();
+                }
+                this.closeAllFloatMenu(menu);
+            }
+        }.bind(this));
+    },
+
+    /**
+     * Устанавливает/отключает видимость элемента меню в соответствии
+     * с текстом для фильтрации
+     * @param filterText Текст для фильтрации
+     * @param menuItem Пункт меню для проверки. Если он содержит дочерние
+     *  элементы, они также проверяются.
+     * @returns {boolean} Стал ли элемент видимым или нет
+     */
+    applyMenuFilter: function (filterText, menuItem) {
+        var text = filterText.toLowerCase(),
+            isVisible = menuItem.text.toLowerCase().indexOf(text) !== -1,
+            currentMenu = menuItem.menu;
+
+        if (currentMenu) {
+            var innerVisibleCount = this.showFilteredStartMenuItems(
+                filterText, currentMenu)
+
+            // Если у вложенного меню внутри есть хотя бы 1
+            // видимый элемент, то это меню тоже видно
+            if (!isVisible && innerVisibleCount > 0) {
+                isVisible = true;
+            }
+
+            if (currentMenu.ul) {
+                currentMenu.ul.setHeight('auto');
+            }
+        }
+        menuItem.setVisible(isVisible);
+        return isVisible;
+    },
+
+    /**
+     * Отфильтровывает пункты меню в соответствии со строкой
+     * @param filterText Текст для фильтрации. Если пустой, фильтр отключается
+     * @param complexMenuItem Пункт меню со вложенными элементами для
+     *  проверки (необязательный параметр). Если не задан, проверяются корневые
+     *  и вложенные в них элементы меню.
+     * @returns {number} Количество видимых элементов внутри данного пункта меню
+     */
+    showFilteredStartMenuItems: function (filterText, complexMenuItem) {
+        var menuItem = complexMenuItem || this,
+            innerVisibleCount = 0;
+
+        if (!filterText) {
+            this.clearMenuSearchFilter();
+            return menuItem.items.length;
+        }
+
+        menuItem.items.each(function (item){
+            innerVisibleCount += Number(this.applyMenuFilter(filterText, item));
+        }.bind(this));
+
+        return innerVisibleCount
+    },
+
+    /**
+     * Делает пункт меню видимым, то есть убирает эффект фильтрации
+     * @param menuItem Пункт меню. Если он содержит дочерние
+     *  элементы, они также обрабатываются.
+     */
+    clearMenuItemSearchFilter: function (menuItem) {
+        if (menuItem.menu) {
+            this.clearMenuSearchFilter(menuItem.menu);
+
+            if (menuItem.menu.ul) {
+                menuItem.menu.ul.setHeight('auto');
+            }
+        }
+        menuItem.setVisible(true);
+    },
+
+    /**
+     * Делает пункты меню видимыми, то есть убирает эффект фильтрации
+     * @param complexMenuItem Пункт меню со вложенными элементами, для которого
+     *  отключается фильтрация. Если не указан параметр, то обрабатываются
+     *  все элементы меню.
+     */
+    clearMenuSearchFilter: function (complexMenuItem) {
+        var menuItem = complexMenuItem || this;
+        menuItem.items.each(this.clearMenuItemSearchFilter.bind(this));
+    },
 });
 
 
@@ -279,7 +431,8 @@ Ext.extend(Ext.ux.TaskBar, Ext.util.Observable, {
             height: 360,
             shadow: true,
             title: 'Jack Slocum',
-            width: 300
+            width: 300,
+            useMenuSearch: this.app.useMenuSearch,
         }, this.app.startConfig));
 
         this.startBtn = new Ext.Button({
@@ -1115,6 +1268,7 @@ Ext.app.App = Ext.extend(Ext.util.Observable, {
         this.enableDragAndDrop = params.enableDragAndDrop;
         this.addDesktopItemUrl = params.addDesktopItemUrl;
         this.removeDesktopItemUrl = params.removeDesktopItemUrl;
+        this.useMenuSearch = params.useMenuSearch;
 
         this.menuItems = desktopConfig.menuItems;
         this.desktopItems = desktopConfig.desktopItems;
@@ -1165,7 +1319,6 @@ Ext.app.App = Ext.extend(Ext.util.Observable, {
                     return true;
                 }
             });
-            this.checkHeights();
         }
     },
 
@@ -1187,10 +1340,7 @@ Ext.app.App = Ext.extend(Ext.util.Observable, {
                 failure: function(){
                     uiAjaxFailMessage.apply(this, arguments);
                 },
-                scope: this,
-                success: function () {
-                    this.checkHeights();
-                },
+                scope: this
             });
         }
     },
@@ -1214,63 +1364,6 @@ Ext.app.App = Ext.extend(Ext.util.Observable, {
         return {}
     },
 
-    checkHeights: function () {
-        // Изменяет размер иконок в зависимости от количества текста
-
-        function addNewHover(cellHeight) {
-            // Изменяет размер картинки заднего плана (block:hover)
-            let head = document.getElementsByTagName('head')[0];
-            let style = document.createElement('style');
-            let declarations = document.createTextNode(
-                `#x-shortcuts td a .block:hover{
-                background:url( ../${params.desktopItemBackgroundUrl} ) no-repeat 0 -${cellHeight}px;
-                background-size: 100% 200%;
-            }`);
-            style.type = 'text/css';
-
-            if (style.styleSheet) {
-              style.styleSheet.cssText = declarations.nodeValue;
-            } else {
-              style.appendChild(declarations);
-            }
-
-            head.appendChild(style);
-        }
-
-        let blocks = Ext.select('.block', true),
-            blockElements = blocks.elements,
-            textHeight = 0;
-        if (blockElements.length === 0) return;
-
-        let cellHeight = blocks.last().getHeight();
-
-        const baseTextHeight = 134,
-              step = 15;
-
-        for (let i = 0; i < blockElements.length; i++) {
-            textHeight = Math.max(textHeight, blockElements[i].query('.text')[0].clientHeight);
-            if (textHeight >= cellHeight) {
-                while (cellHeight <= textHeight) {
-                    cellHeight += step;
-                }
-            } else if (baseTextHeight >= textHeight) {
-                while (cellHeight > baseTextHeight && cellHeight > 149) {
-                    cellHeight -= step;
-                }
-            }
-        }
-
-        Ext.each(blockElements, function (block) {
-            block.setStyle({
-                'background-size': '100% 200%',
-            })
-            block.select('.base-desktop-image').item(0).setStyle({
-                'height': `${cellHeight}px`
-            })
-        });
-        addNewHover(cellHeight);
-    },
-
     removeDesktopItem: function(id){
         var el = Ext.get(id),
             cmp = getDesktopComponent(el);
@@ -1286,10 +1379,7 @@ Ext.app.App = Ext.extend(Ext.util.Observable, {
                 failure: function(){
                     uiAjaxFailMessage.apply(this, arguments);
                 },
-                scope: this,
-                success: function () {
-                    this.checkHeights();
-                },
+                scope: this
             });
         }
     },
