@@ -30,8 +30,17 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
      */
     ,iconClsDownloadFile: 'x-form-file-download-icon'
 
+    /**
+     * Множественный выбор файлов
+     */
+    ,multiple: false
+
     ,constructor: function(baseConfig, params){
         if (params) {
+            this.hideClearButton = params.hideClearButton || false;
+            this.hideUploadButton = params.hideUploadButton || false;
+            this.hideDownloadButton = params.hideDownloadButton || false;
+
             if (params.prefixUploadField) {
                 this.prefixUploadField = params.prefixUploadField;
             }
@@ -44,6 +53,9 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
             if (params.possibleFileExtensions) {
                 this.possibleFileExtensions = params.possibleFileExtensions;
             }
+            if (params.multiple) {
+                this.multiple = params.multiple;
+            }
         }
 
         Ext.ux.form.FileUploadField.superclass.constructor.call(this, baseConfig, params);
@@ -99,6 +111,7 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
             ,tooltip: {
                 text:'Выбрать файл'
                 ,width: 150
+            ,hidden: this.hideUploadButton
             }
         }));
 
@@ -109,7 +122,7 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
             ,iconCls: this.iconClsClearFile
             ,handler: this.clickClearField
             ,scope: this
-            ,hidden: this.value ? false : true
+            ,hidden: !(this.value && !this.hideClearButton)
             ,tooltip: {
                 text:'Очистить'
                 ,width: 65
@@ -140,7 +153,7 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
             ,iconCls: this.iconClsDownloadFile
             ,handler: this.clickDownload
             ,scope: this
-            ,hidden: this.value ? false : true
+            ,hidden: !(this.value && !this.hideDownloadButton)
              ,tooltip: {
                 text:'Загрузить'
                 ,width: 65
@@ -178,7 +191,16 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
                      this.reset();
                      return;
                  }
-                 var v = this.fileInput.dom.value;
+                 if (this.multiple) {
+                     var v,
+                         filenames = [];
+                     Ext3.each(this.fileInput.dom.files, function (item) {
+                         filenames.push(item.name);
+                     });
+                     v = filenames.join(';')
+                 } else {
+                     var v = this.fileInput.dom.value;
+                 }
                  if (this.fireEvent('beforechange', this, v)) {	                 		                 
 	                 this.setValue(v);
 	                 this.fireEvent('fileselected', this, v);
@@ -199,7 +221,7 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
     }
 
     ,createFileInput : function() {
-        this.fileInput = this.wrap.createChild({
+        var fileInputParams = {
             id: this.getFileInputId(),
             name: (this.prefixUploadField || '') + this.name,
             cls: 'x-form-file',
@@ -207,7 +229,11 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
             type: 'file',
             size: 1,
             width: 20
-        });
+        };
+        if (this.multiple) {
+            fileInputParams['multiple'] = this.multiple;
+        }
+        this.fileInput = this.wrap.createChild(fileInputParams);
 
         Ext.QuickTips.unregister(this.fileInput);
         Ext.QuickTips.register({
@@ -246,13 +272,7 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
             w -= btnDonwloadWidth;
         }
 
-        if (Ext.isWebKit) {
-            // Юлядть
-            // Некорректная верстка в вебкитовских движках
-            this.el.setWidth(w + 5);
-        } else {
-            this.el.setWidth(w);
-        }
+        this.el.setWidth(w);
 
     }
 
@@ -338,6 +358,18 @@ Ext.ux.form.FileUploadField = Ext.extend(Ext.form.TextField,  {
     ,setReadOnly: function(readOnly){
          Ext.ux.form.FileUploadField.superclass.setReadOnly.call(this, readOnly);
     }
+    //override
+    ,validate: function() {
+        var isValid = Ext.ux.form.FileUploadField.superclass.validate.call(this);
+        if (!this.allowBlank){
+            if (isValid) {
+                this.addClass('m3-grey-field');
+            } else {
+                this.removeClass('m3-grey-field');
+            }
+        }
+        return isValid
+    }
 });
 
 Ext.reg('fileuploadfield', Ext.ux.form.FileUploadField);
